import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;

/**
   This class demonstrates use of the Swing text pane.  Text panes can
   simultaneously parse and display multiple styles of text.
*/
class TextPaneDemo extends JPanel
{
  /**
     No-arg constructor.
  */
  public TextPaneDemo()
  {
    Font f14 = new Font("SansSerif", Font.PLAIN, 14);
    Font if14 = new Font("SansSerif", Font.ITALIC, 14);

    Dimension dim = new Dimension(300, 30);

    String newline = System.getProperty("line.separator");

    // Create a text pane object.  
    //
    JTextPane textPane1 = new JTextPane();

    // Call the helper methods defined in this class to do all the
    // intialization of styles.
    //
    initializeStyles(textPane1);
    setPaneText(textPane1);

    // Place the text pane in a scroll pane so you can scroll its
    // contents.
    //
    JScrollPane scrollPane1 = new JScrollPane(textPane1);
    scrollPane1.setVerticalScrollBarPolicy(
        JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
    scrollPane1.setHorizontalScrollBarPolicy(
        JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
    scrollPane1.setPreferredSize(new Dimension(450, 225));

    Box box = Box.createVerticalBox();
    box.add(scrollPane1);
    add(box);
  }


  // This method creates a set of styles to use for rendering the text
  // that will be placed in the text pane.
  //
  private void initializeStyles(JTextPane pane)
  {
    StyleContext context = StyleContext.getDefaultStyleContext();
    Style defaultStyle = context.getStyle(StyleContext.DEFAULT_STYLE);

    // The addStyle() method actually creates a new style.  After
    // creating a new style you have to set the attribute values so
    // that the style object represents the desired style.  When you
    // create the style, choose a name that matches the style you
    // configure by setting the attribute values.  

    // Styles are associated in a hierarchy.  If an attribute can't be
    // found in a style, its parent is searched.  The second argument
    // is the parent.  
    //
    Style newStyle = pane.addStyle("normal", defaultStyle);
    StyleConstants.setFontFamily(newStyle, "SansSerif");
    StyleConstants.setFontSize(newStyle, 16);

    // Create another new style.  Set the style to italic.
    // The default size is 14 points.
    //
    newStyle = pane.addStyle("italic-14", defaultStyle);
    StyleConstants.setItalic(newStyle, true);
    
    // Create a new style.  Set the style to italic, the font to "serif",
    // and the point size to 16.
    //
    newStyle = pane.addStyle("italic-16", defaultStyle);
    StyleConstants.setItalic(newStyle, true);
    StyleConstants.setFontFamily(newStyle, "Serif");
    StyleConstants.setFontSize(newStyle, 16);
    
    // Create a style with family monospaced and point size 10.
    //
    newStyle = pane.addStyle("monospaced", defaultStyle);
    StyleConstants.setFontFamily(newStyle, "Monospaced");
    StyleConstants.setFontSize(newStyle, 18);

    // Create a new style with bold style and point size 16.
    //
    newStyle = pane.addStyle("bold", defaultStyle);
    StyleConstants.setBold(newStyle, true);
    StyleConstants.setFontSize(newStyle, 16);

    // You get the idea.
    //
    newStyle = pane.addStyle("serif-14", defaultStyle);
    StyleConstants.setFontFamily(newStyle, "Serif");
    StyleConstants.setFontSize(newStyle, 14);

    newStyle = pane.addStyle("serif-16", defaultStyle);
    StyleConstants.setBold(newStyle, false);
    StyleConstants.setFontFamily(newStyle, "Serif");
    StyleConstants.setFontSize(newStyle, 16);

    newStyle = pane.addStyle("bold-italic-18", defaultStyle);
    StyleConstants.setBold(newStyle, true);
    StyleConstants.setItalic(newStyle, true);
    StyleConstants.setFontSize(newStyle, 18);
  }


  // This helper method sets runs of text in the text pane.  Each run
  // of text must be associated with one of the styles created in the
  // initializeStyles() method.
  //
  private void setPaneText(JTextPane pane)
  {
    // Get the document for the text pane created earlier.  The
    // document is a model, an object that abstracts the structure of
    // the text pane and stores its contents.
    //
    Document document = pane.getDocument();

    try
      {
	// Create a run of text to be rendered in one style.  Then
	// insert the text in the text pane's document.  At this time
	// you must associate the text with the style in which it will
	// be rendered.
	//
	StringBuffer str = new StringBuffer("In addition to the ");
	document.insertString(document.getLength(), str.toString(),
			      pane.getStyle("normal"));
    
	// Create the next run of text to be inserted in the text
	// pane.  Again, associate it with the desired style.  
	// 
	str = new StringBuffer("JEditorPane");
	document.insertString(document.getLength(), str.toString(),
			      pane.getStyle("monospaced"));
    
	// Repeat the process.  Notice you have to specify a position
	// in the document for the text insertion.  While initially
	// filling the text pane, the position is one more than the
	// position of the last character inserted (the current length
	// of the document).
	//
	str = new StringBuffer(" class ");
	document.insertString(document.getLength(), str.toString(),
			      pane.getStyle("normal"));
    
	// Repeat the process for each run of text whose style differs
	// from the previous style.
	//
	str = new StringBuffer("the Swing Tool Kit ");
	document.insertString(document.getLength(), str.toString(),
			      pane.getStyle("italic-16"));
    
	str = new StringBuffer("defines another class that can do some ");
	document.insertString(document.getLength(), str.toString(),
			      pane.getStyle("serif-14"));
    
	str = new StringBuffer("pretty fancy ");
	document.insertString(document.getLength(), str.toString(),
			      pane.getStyle("bold"));

	str = new StringBuffer("text formatting.  ");
	document.insertString(document.getLength(), str.toString(),
			      pane.getStyle("serif-14"));
	
	str = new StringBuffer("Instances of the ");
	document.insertString(document.getLength(), str.toString(),
			      pane.getStyle("serif-14"));
    
	str = new StringBuffer("JTextPane");
	document.insertString(document.getLength(), str.toString(),
			      pane.getStyle("monospaced"));

	str = new StringBuffer(" class can present runs of text which are ");
	document.insertString(document.getLength(), str.toString(),
			      pane.getStyle("normal"));

	str = new StringBuffer("each ");
	document.insertString(document.getLength(), str.toString(),
			      pane.getStyle("bold"));

	str = new StringBuffer("presented in a ");
	document.insertString(document.getLength(), str.toString(),
			      pane.getStyle("italic-14"));

	str = new StringBuffer("different style.");
	document.insertString(document.getLength(), str.toString(),
			      pane.getStyle("bold-italic-18"));

	str = new StringBuffer("  To use this capability you must ");
	document.insertString(document.getLength(), str.toString(),
			      pane.getStyle("serif-14"));

	str = new StringBuffer("first create");
	document.insertString(document.getLength(), str.toString(),
			      pane.getStyle("bold"));

	str = new StringBuffer(" the ");
	document.insertString(document.getLength(), str.toString(),
			      pane.getStyle("normal"));

	str = new StringBuffer("different styles,");
	document.insertString(document.getLength(), str.toString(),
			      pane.getStyle("bold-italic-18"));

	str = new StringBuffer(" then add them to the ");
	document.insertString(document.getLength(), str.toString(),
			      pane.getStyle("normal"));

	str = new StringBuffer("JTextPane,");
	document.insertString(document.getLength(), str.toString(),
			      pane.getStyle("monospaced"));

	str = new StringBuffer(" and finally ");
	document.insertString(document.getLength(), str.toString(),
			      pane.getStyle("serif-14"));

	str = new StringBuffer("insert the text!  ");
	document.insertString(document.getLength(), str.toString(),
			      pane.getStyle("italic"));
      }
    catch (BadLocationException e)
      {
	// Catches exceptions caused by specifying a bad position
	// in the document model.
	//
      }

  }


  public static void main(String [] args)
  {
    JFrame f = new JFrame("Text Pane Demo");
    WindowListener wL = new WindowAdapter()
      {
	public void windowClosing(WindowEvent e)
	  {
	    ((Window) e.getSource()).dispose();
	    System.exit(0);
	  }
      };
    f.addWindowListener(wL);

    TextPaneDemo app = new TextPaneDemo();

    f.getContentPane().add(app);
    f.pack();
    f.setVisible(true);
  }
}
